#include "master.h"
using namespace std;
CmeanField *CmasterSampler::meanfield=NULL;

CmasterSampler::CmasterSampler(CparameterMap *parmapin){
	parmap=parmapin;
	randy=new Crandy(-1234);
	reslist=new CresList(parmap);	
	NEVENTS=0;
	RESWIDTH_ALPHA=parmap->getD("SAMPLER_RESWIDTH_ALPHA",0.5);
	NTF=parmap->getI("SAMPLER_NTF",1);
	NSIGMAF=parmap->getI("SAMPLER_NSIGMAF",1);
	TFmin=parmap->getD("SAMPLER_TFMIN",155.0);
	TFmax=parmap->getD("SAMPLER_TFMAX",155.0);
	SIGMAFmin=parmap->getD("SAMPLER_SIGMAFMIN",93.0);
	SIGMAFmax=parmap->getD("SAMPLER_SIGMAFMAX",93.0);
	SETMU0=parmap->getB("SAMPLER_SETMU0",false);
	if(NTF==1)
		DELTF=0.0;
	else
		DELTF=(TFmax-TFmin)/double(NTF);
	if(NSIGMAF==1)
		DELSIGMAF=0.0;
	else
		DELSIGMAF=(SIGMAFmax-SIGMAFmin)/double(NSIGMAF);
	Csampler::randy=randy;
	Csampler::mastersampler=this;
	Csampler::reslist=reslist;
	Csampler::parmap=parmap;
	Cpart::reslist=reslist;
	int it,isigma;
	sampler.resize(NTF);
	for(it=0;it<NTF;it++){
		sampler[it].resize(NSIGMAF);
		for(isigma=0;isigma<NSIGMAF;isigma++){
			sampler[it][isigma]=new Csampler(TFmin+(it+0.5)*DELTF,SIGMAFmin+isigma*DELSIGMAF);
		}
	}
}

CmasterSampler::~CmasterSampler(){
	//printf("MasterSampler ending, NEVENTS=%d\n",NEVENTS);
}

int CmasterSampler::MakeEvent(){
	int np;
	Csampler *sampler;
	double Omega0Sum=0.0;
	int nelements=hyper.size();
	int ielement,nparts=0;
	for(ielement=0;ielement<nelements;ielement++){
		Omega0Sum+=hyper[ielement]->dOmega[0];
		np=0;
		sampler=ChooseSampler(hyper[ielement]);
		if(!SETMU0){
			if(NEVENTS==0){
				sampler->GetTfMuNH(hyper[ielement]);
				sampler->CalcLambdaF();
				hyper[ielement]->muB=sampler->muB;
				hyper[ielement]->muI=sampler->muI;
				hyper[ielement]->muS=sampler->muS;
				hyper[ielement]->nhadrons=sampler->nhadronsf;
				hyper[ielement]->epsilon=sampler->epsilonf;
				hyper[ielement]->lambda=sampler->lambdaf;
			}
			else{
				sampler->muB=hyper[ielement]->muB;
				sampler->muI=hyper[ielement]->muI;
				sampler->muS=hyper[ielement]->muS;
				sampler->nhadronsf=hyper[ielement]->nhadrons;
				sampler->epsilonf=hyper[ielement]->epsilon;
				sampler->lambdaf=hyper[ielement]->lambda;
				sampler->Pf=sampler->nhadronsf*sampler->Tf;
			}
		}
		np+=sampler->MakeParts(hyper[ielement]);
		nparts+=np;
	}
	NEVENTS+=1;
	return nparts;
}

Csampler* CmasterSampler::ChooseSampler(Chyper *hyper){
	double T,sigma,rhoB,rhoI,T0,T1;
	int it,isigma;
	if(NTF==1){
		it=0;
	}
	else{
		T=hyper->T;
		it=lrint((T-TFmin-0.5*DELTF)/DELTF);
		if(it<0)
			it=0;
		if(it>=NTF)
			it=NTF-1;
	}
	if(NSIGMAF==1){
		isigma=0;
	}
	else{
		sigma=hyper->sigma;
		isigma=lrint((sigma-SIGMAFmin-0.5*DELSIGMAF)/DELSIGMAF);
		if(isigma<0)
			isigma=0;
		if(isigma>=NSIGMAF)
			sigma=NSIGMAF-1;
	}
	return sampler[it][isigma];
}

/*
void CmasterSampler::CalcPiFromParts(){
	double pi[4][4]={0.0};
	CpartMap::iterator ppos;
	CPart *part;
	double *p,pressure,Ptest=0.0,etest=0.0;
	int ipart,alpha,beta;
	double volume=nelements*hyper[0].Omega[0];
	for(ppos=b3d->DeadPartMap.begin();ppos!=b3d->PartMap.end();ppos++){
		part=ppos->second;
		pressure=(part->p[1]*part->p[1]+part->p[2]*part->p[2]+part->p[3]*part->p[3])/(3.0*part->p[0]);
		Ptest+=pressure;
		etest+=p[0];
		for(alpha=1;alpha<4;alpha++){
			pi[alpha][alpha]-=pressure;
			for(beta=1;beta<4;beta++){
				pi[alpha][beta]+=p[alpha]*p[beta]/p[0];
			}
		}
	}
	printf("----From particle momenta  -----\n");
	printf("epsilon=%10.4f, P=%10.4f\n",etest/volume,Ptest/volume);
	printf("pi_ij/P\n");
	for(alpha=1;alpha<4;alpha++){
		for(beta=1;beta<4;beta++){
			pi[alpha][beta]=pi[alpha][beta]/volume;
			printf("%9.6f ",pi[alpha][beta]/hyper[0].P);
		}
		printf("\n");
	}
	printf("---- From pi_ij  input -------\n");
	printf("epsilon=%10.4f, P=%10.4f\n",hyper[0].epsilon,hyper[0].P);
	printf("pi_ij/P\n");
	for(alpha=1;alpha<4;alpha++){
		for(beta=1;beta<4;beta++){
			printf("%9.6f ",hyper[0].pitilde[alpha][beta]/hyper[0].P);
		}
		printf("\n");
	}
}
*/

void CmasterSampler::ReadHyper2D(string filename){

}



